// ***********************************************************
// Project: lcd HD44780 4-bit
// Author:
// : 1.1 
//    bv (   rw     )
// ***********************************************************
#include <avr/io.h>              // Most basic include files
#include <avr/interrupt.h>       // Add the necessary ones
#include <util/delay.h> /*F_CPU = 4000000; _delay_ms max = 65,535ms; _delay_us max = 192us */
#include <avr/pgmspace.h>
#include "lph.h"


const unsigned char lcdaddr[4] = {0X80, 0XC0, 0X94, 0XD4};

void lcd_4bus_write(uint8_t data){//   4  
	PORTLCD&=0xF0;
	DDRLCD|=0x7F;
	
	PORTLCD|=E;
	PORTLCD|=(data>>4)&0x0F;
	PORTLCD&=~E;

	PORTLCD&=0xF0;
	PORTLCD|=E;
	PORTLCD|=data&0x0F;
	PORTLCD&=~E;

	DDRLCD&=0xF0;
	PORTLCD&=0x80;
	//RS=RW=0,  DB3..0 = 0    Z   
}

uint8_t lcd_read_byte(void){//   4   
	//  .  PORTLCD 4      
	//     
uint8_t byte;
	DDRLCD&=0xF0;// RS|RW|E|0x80
	PORTLCD|=RW;
		//sig_E();
	PORTLCD|=E;	
	_delay_us(5);//5us
	byte=( PINLCD & 0x0F )<<4;
	PORTLCD&=~E;
		//sig_E();
	
	PORTLCD|=E;
	_delay_us(5);//5us
	byte|=PINLCD&0x0F;
	PORTLCD&=~E;
	PORTLCD&=0x80;//     
	return byte;
	//  , RS=RW=0,  DB3..0 = 0    Z  
}

uint8_t read_busy_address(void){
uint8_t byte;
	
	PORTLCD&=0x80;
	while( (uint8_t) (( byte=lcd_read_byte() ) & busy_flag) );//   busy_flag
	return byte;
	//   AC, RS=RW=0,  DB3..0 = 0    Z  

}

void lcd_write_data(uint8_t data){
	//write data to CGRAM or DDRAM
	read_busy_address();//PORTLCD=X0000000
	PORTLCD|=RS;//RS = 1 -   (DR). RW = 0 - 
	lcd_4bus_write(data);
}

uint8_t lcd_read_data(void){
	//read data from CGRAM or DDRAM
	read_busy_address();////   AC, RS=RW=0,  DB3..0 = 0    Z 
	PORTLCD|=RS|RW; //RS = 1 -   (DR). RW = 1 - 
	return lcd_read_byte();//byte;
}

void lcd_instructions_set(uint8_t address){
	//   -> address|( , cgram  ddram)
	read_busy_address();//   AC, RS=RW=0,  DB3..0 = 0    Z  
	lcd_4bus_write(address);
}


void lcd_set_address(uint8_t x, uint8_t y){
	lcd_instructions_set(lcdaddr[y-1] + x);
}

void lcd_init(void){

//	PORTLCD = 0x80;
//	DDRLCD = 0x7F;//0b11110011;    // output
//	_delay_ms(60);
//	lcd_instructions_set(0x30);      
//	_delay_ms(5);
//	lcd_instructions_set(0x30);      
//	_delay_us(200);
//	lcd_instructions_set(0x30);      
//	_delay_us(200);
//	lcd_instructions_set(0x02);      // 4bit mode
//	lcd_instructions_set(0x02);      // 4bit mode
//	lcd_instructions_set(0x28);       // 4 bit, 2 lines, 5*7 font
//	lcd_instructions_set(0x0C);      // cursor OFF, display ON
//	lcd_instructions_set(0x06);      // -> inc
//	lcd_instructions_set(0x01);
	PORTLCD&=0x80;
	DDRLCD|=0x7F;
	
	_delay_ms(40);//   20 (   5)
	
	PORTLCD|=E;
	PORTLCD|=DB5|DB4;			//function set
	PORTLCD&=~E;
		
	_delay_ms(5); //2ms
	
	PORTLCD|=E;
	PORTLCD|=DB5|DB4;			//function set
	PORTLCD&=~E;
	_delay_us(50);//40us
	PORTLCD|=E;
	PORTLCD|=DB5|DB4;			//function set
	PORTLCD&=~E;
	_delay_us(50);//40us
	
	PORTLCD&=0x80;
	PORTLCD|=E;
	PORTLCD|=DB5;				//function set 4bit
	PORTLCD&=~E;
	
	_delay_us(50);//40us
	lcd_instructions_set(0x2A);
	lcd_instructions_set(0x08);
	lcd_instructions_set(0x01);
	lcd_instructions_set(0x06);
	// Initialization ends
	
	lcd_instructions_set(cursor_off);

 
 }


//****************************************************************************************************

	void LCDwritebyte(uint8_t a, uint8_t pc)//      LCD
	{
	lcd_instructions_set(a);
	lcd_write_data(pc);
	}
	void LCDdefinechar(const uint8_t *pc,uint8_t char_code){ //    FLASHROM
	uint8_t a, pcc;
	uint16_t i;
	a=(char_code<<3)|0x40;
	for (i=0; i<8; i++){
		pcc=pgm_read_byte(&pc[i]);
		LCDwritebyte(a++,pcc);
		}
}





